/**
 * BLOCK: my-block
 *
 * Registering a basic block with Gutenberg.
 * Simple block, renders and saves the same content without any interactivity.
 */

import CJTBlockList from './comp/CJTBlockList';

const { __ } = wp.i18n;
const { registerBlockType } = wp.blocks;

/**
 * Register: aa Gutenberg Block.
 *
 * Registers a new block provided a unique name and an object defining its
 * behavior. Once registered, the block is made editor as an option to any
 * editor interface where blocks are implemented.
 *
 * @link https://wordpress.org/gutenberg/handbook/block-api/
 * @param  {string}   name     Block name.
 * @param  {Object}   settings Block settings.
 * @return {?WPBlock}          The block, if it has been successfully
 *                             registered; otherwise `undefined`.
 */

registerBlockType( 'cjt-plus/cjt-cb-list', {
	title: __( 'CJT Code Blocks' ),
	icon: CJTBlockList.icon,
	category: 'common',
	keywords: [
		__( 'css-javascript-toolbox' ),
		__( 'cjt' ),
		__( 'cjt-gutenberg' ),
	],

	attributes: {
		content: {
			type: 'string',
			selector: 'p',
		},
		name: {
			type: 'string',
			selector: 'h4',
		},
		selectedPost: {
			type: 'number',
			default: 0,
		},
	},

	edit: CJTBlockList,

	save: function( props ) {
		return (
			<div className={ props.className }>
				<div className="post">
					<p dangerouslySetInnerHTML={ { __html: props.attributes.content } }></p>
				</div>
			</div>
		);
	},
} );
