const { __ } = wp.i18n;
const { Component } = wp.element;
const el = wp.element.createElement;
const { SelectControl } = wp.components;
const { InspectorControls } = wp.editor;

export default class CJTBlockList extends Component {
	static icon = function() {
		return el( 'svg',
			{
				width: 20,
				height: 20,
				viewBox: '0 0 20 20',
			},
			el( 'path',
				{
					d: 'M 2.34375 2.316406 C 2.34375 2.316406 1.5625 2.316406 1.5625 3.097656 L 1.5625 8.566406 C 1.5625 8.566406 1.5625 9.347656 2.34375 9.347656 L 17.660156 9.347656 C 17.660156 9.347656 18.441406 9.347656 18.441406 8.566406 L 18.441406 3.097656 C 18.441406 3.097656 18.441406 2.316406 17.660156 2.316406 Z M 2.34375 2.316406',
				},
			),
			el( 'path',
				{
					d: 'M 20.003906 17.003906 L 20.003906 11.535156 C 20.003906 11.535156 20.003906 10.753906 19.222656 10.753906 L 11.015625 10.753906 L 11.015625 12.628906 C 11.015625 13.410156 10.234375 13.410156 10.234375 13.410156 L 9.765625 13.410156 C 8.984375 13.410156 8.984375 12.628906 8.984375 12.628906 L 8.984375 10.753906 L 0.78125 10.753906 C 0.78125 10.753906 0 10.753906 0 11.535156 L 0 17.003906 C 0 17.003906 0 17.789062 0.78125 17.789062 L 19.222656 17.789062 C 19.222656 17.789062 20.003906 17.789062 20.003906 17.003906 Z M 20.003906 17.003906',
				}
			)
		);
	}

	static getInitialState( selectedPost ) {
		return {
			posts: [],
			selectedPost: selectedPost,
			post: {},
		};
	}

	getOptions() {
		let _url = location.origin + location.pathname;
		_url = _url.split( '/wp-admin/' )[ 0 ] + '/wp-json/cjt-plus/v1/blocks';
		return fetch( _url )
			.then( r => r.json() )
			.then( ( posts ) => {
				if ( posts && 0 !== this.state.selectedPost ) {
					const post = posts.find( ( item ) => {
						return parseInt( item.id ) === parseInt( this.state.selectedPost );
					} );
					this.setState( { post, posts } );
				} else {
					this.setState( { posts } );
				}
			} );
	}

	onChangeSelectPost( value ) {
		if ( value <= 0 ) {
			return false;
		}
		// Find the post
		const post = this.state.posts.find( ( item ) => {
			return parseInt( item.id ) === parseInt( value );
		} );

		// Set the state
		this.setState( { selectedPost: parseInt( value ), post } );
		// Set the attributes
		this.props.setAttributes( {
			selectedPost: parseInt( value ),
			name: post.name,
			content: '[cjtoolbox name="' + post.name + '"]',
		} );
	}

	constructor() {
		super( ...arguments );
		this.state = this.constructor.getInitialState( this.props.attributes.selectedPost );
		this.getOptions = this.getOptions.bind( this );
		this.getOptions();

		this.onChangeSelectPost = this.onChangeSelectPost.bind( this );
	}

	render() {
		const options = [ { value: 0, label: __( 'Select a Code Block' ) } ];
		let output = __( 'Loading Code Blocks...' );
		this.props.className += ' loading';
		if ( this.state.posts.length === 1 ) {
			output = __( 'One CJT Code Block was found. Please select it using the dropdown field.' );

			this.state.posts.forEach( ( post ) => {
				options.push( { value: post.id, label: post.name } );
			} );
		} else if ( this.state.posts.length > 1 ) {
			const loading = __( '%d CJT Code Blocks were found. Please select one using the dropdown field.' );
			output = loading.replace( '%d', this.state.posts.length );

			this.state.posts.forEach( ( post ) => {
				options.push( { value: post.id, label: post.name } );
			} );
		} else {
			output = __( 'No CJT Code Blocks were found. Please create one or more first, then refresh this page.' );
		}
		// Checking if we have anything in the object
		if ( this.state.post.hasOwnProperty( 'name' ) ) {
			output = <div className="post">
				<h4 dangerouslySetInnerHTML={ { __html: 'CJT Code Block: ' + this.state.post.name } }></h4>
			</div>;
			this.props.className += ' has-post';
		} else {
			this.props.className += ' no-post';
		}

		return [
			<InspectorControls key="inspector">
				<SelectControl onChange={ this.onChangeSelectPost } value={ this.props.attributes.selectedPost } label={ __( 'Select a Code Block' ) } options={ options } />
			</InspectorControls>,
			<div key={ 1 }>{ output }</div>,
		];
	}
}
